#ifndef TCFUSIONZOOM_H
#define TCFUSIONZOOM_H

#include <inttypes.h>

class TFC
{

public:

    //! TMC data
    /*!
     * Pointer to the compressed
     * TMC data (infra red) of
     * a TFC frame.
     */
    unsigned char* irData;

    //! TMC data size
    /*!
     * Size of the TMC data.
     */
    unsigned int irDataSize;

    //! Jpeg data
    /*!
     * Pointer to jpeg binary data.
     */
    unsigned char* jpegData;

    //! Jpeg data size
    /*!
     * Size of the jpeg binary data.
     */
    unsigned int jpegDataSize;

    //! Constructor of TFC
    /*!
     * Initializes an object of type TFC.
     * It holds the pointer to TMC and jpeg data
     * with the appropriate size infos.
     */
    TFC();

    //! Destructor of TFC.
    /*!
    *   Cleans up the TFC object memory.
    */
    ~TFC();
};



//! Class for connecting "TCFusionZoom".
/*!
*   Connecting TCFusionZoom needs three steps:
*   1. Include the library header file "tcfusionzoom.h" into your project.
*   2. Write a callback function for receiving TFC frames.
*   3. Use the constructor of TCFusionZoom. You will have to put in 2 parameters.
*   First put in the pointer of the callback function.
*   Second add the a pointer of the calling instance in the constructor.
*/
class TCFusionZoom
{

public:

    //! Type definition of callback.
    typedef void (*callbackTFC)(TFC& tfc, void* caller);

    //! Constructor of TCFusionZoom.
    /*!
    *   TCFusionZoom is used to connect the "Thermal Capture Fusion Zoom" device.
    *   The callback is called every time a full frame is received from the device.
    *   Calling instance is needed for accessing the caller object's function from
    *   the callback.
    *   \param cb Callback.
    *   \param caller Calling instance.
     */
    TCFusionZoom(callbackTFC cb, void* caller);


    //! Destructor of TCFusionZoom.
    /*!
    *   Cleans up the TCFusionZoom object memory and waits for
    *   connection thread to close.
    */
    ~TCFusionZoom();

    //! Enable usb TFC stream.
    /*!
    *   Enables the usb TFC data stream
    *   from TCFusionZoom to the lib.
    */
    void sendCmdUSBTransferOutOn();

    //! Disables usb TFC stream.
    /*!
    *   Disables the usb TFC stream
    *   from TCFusionZoom to the lib.
    */
    void sendCmdUSBTransferOutOff();

    //! Trigger FFC.
    /*!
    *   Triggers the TauCore to make an FFC.
    */
    void sendCmdFFC();

    //! Set zoom level.
    /*!
    *   Sets the zoom level. Valid values 1x - 10x.
    *   \param zoomLevel to set
    */
    void sendCmdZoom(int zoomLevel);


    void reset();

private:

    // internally used reference to callback and calling instance
    callbackTFC mCallbackTFC;
    void* mCallingInstance;

    // internally used callback definitions
    static void WrapperCallbackTCFusionZoomService(TFC& tfc, void* caller);
    void callbackTCFusionZoomService(TFC& tfc);
};

#endif // TCFUSIONZOOM_H
